#pragma once

#include "ofMain.h"

//uuid
#include "random_id.h"

#include "ofxMsdfgen.h"
#include "Utils.h"
#include <memory>

namespace ofxVariableLab {

struct LayerSettings {
    uint32_t maxBufferSize = 100;
    VFlipBehaviour vFlipBehaviour = V_FLIP_ONCE_AUTO;
};

class Layer {
    public:
        enum TransformOrigin : int {
            TOP_LEFT = 0,
            TOP_RIGHT,
            CENTER,
            BOTTOM_LEFT,
            BOTTOM_RIGHT
        };
        struct Props {
            float x = 200;
            float y = 200;
            float rotation = 0;
            float fontSize_px = 42;
            float color[4];
            bool mirror_x = false;
            float mirror_x_distance = 0;
            bool mirror_y = false;
            float mirror_y_distance = 0;
            bool mirror_xy = false;
            float letterDelay = 0;
            int transformOrigin = TransformOrigin::TOP_LEFT;
            std::vector <ofxVariableLab::FontVariation> fontVariations;
            string text = "abc";
            string fontPath = "data/celines-fonts/Version-2-var.ttf";
            //NLOHMANN_DEFINE_TYPE_NON_INTRUSIVE(Props,
            //x,
            //y,
            //rotation,
            //fontSize_px,
            //color,
            //mirror_x,
            //mirror_x_distance,
            //mirror_y,
            //mirror_y_distance,
            //letterDelay,
            //fontVariations,
            //text,
            //fontPath)
        };

        virtual void setup(const LayerSettings & settings = LayerSettings()) = 0;
        virtual void update() = 0;
        virtual void draw(glm::vec3 position = glm::vec3(0, 0, 0)) = 0;
        virtual void drawCharacter(const char character,
                                   glm::vec3 position = glm::vec3(0, 0, 0),
                                   glm::vec4 color = glm::vec4(1, 1, 1, 1),
                                   float rotation = 0,
                                   float scale = 1,
                                   ofxVariableLab::FontVariation fontVariation = ofxVariableLab::FontVariation()) = 0;
        virtual void setProps(const Props & props) = 0;
        virtual const Props & getProps(float delay = 0) const = 0;
        virtual const glm::vec4 & getColor() const = 0;
        virtual void clearPropsBuffer() = 0;
        virtual void setId(const LayerID & id) = 0;
        virtual const LayerID & getId() = 0;
        virtual void setShader(shared_ptr <ofShader> _shader) = 0;
        virtual shared_ptr <ofShader> getShader() const = 0;
        virtual const LayerType & getType() const = 0;
        virtual void setVFlip(const VFlipState vFlip) = 0;
        virtual bool isDirtyDirty() const = 0;
        virtual void setDirtyDirty(bool dirtyDirty) = 0;
        virtual bool wantsNewMom() = 0;
        virtual void isWithNewMom() = 0;
        virtual const ComboIdentifier & getMomsComboIdentifier() const = 0;
        virtual void setMomsComboIdentifier(const ComboIdentifier & identifier) = 0;
        virtual ofNode & getOuterNode() = 0;
        virtual ofNode & getInnerNode() = 0;
        static int n_layers;
};
}
